package com.sun.corba.se.impl.io;

import org.omg.CORBA.portable.ValueOutputStream;
import java.io.ObjectOutput;
import java.util.HashMap;
import java.util.Map;
import org.omg.CORBA_2_3.portable.OutputStream;
import java.io.NotActiveException;
import java.io.IOException;
import java.io.ObjectOutputStream;

public abstract class OutputStreamHook extends ObjectOutputStream
{
    private OutputStreamHook.HookPutFields putFields;
    protected byte streamFormatVersion;
    protected OutputStreamHook.WriteObjectState writeObjectState;
    protected static final OutputStreamHook.WriteObjectState NOT_IN_WRITE_OBJECT;
    protected static final OutputStreamHook.WriteObjectState IN_WRITE_OBJECT;
    protected static final OutputStreamHook.WriteObjectState WROTE_DEFAULT_DATA;
    protected static final OutputStreamHook.WriteObjectState WROTE_CUSTOM_DATA;
    
    abstract void writeField(ObjectStreamField p0, Object p1) throws IOException;
    
    public OutputStreamHook() throws IOException {
        this.putFields = null;
        this.streamFormatVersion = 1;
        this.writeObjectState = OutputStreamHook.NOT_IN_WRITE_OBJECT;
    }
    
    @Override
    public void defaultWriteObject() throws IOException {
        this.writeObjectState.defaultWriteObject(this);
        this.defaultWriteObjectDelegate();
    }
    
    public abstract void defaultWriteObjectDelegate();
    
    @Override
    public ObjectOutputStream.PutField putFields() throws IOException {
        if (this.putFields == null) {
            this.putFields = new OutputStreamHook.HookPutFields();
        }
        return this.putFields;
    }
    
    public byte getStreamFormatVersion() {
        return this.streamFormatVersion;
    }
    
    abstract ObjectStreamField[] getFieldsNoCopy();
    
    @Override
    public void writeFields() throws IOException {
        this.writeObjectState.defaultWriteObject(this);
        if (this.putFields != null) {
            this.putFields.write(this);
            return;
        }
        throw new NotActiveException("no current PutField object");
    }
    
    abstract org.omg.CORBA_2_3.portable.OutputStream getOrbStream();
    
    protected abstract void beginOptionalCustomData();
    
    protected void setState(OutputStreamHook.WriteObjectState writeObjectState) {
        this.writeObjectState = writeObjectState;
    }
    
    static {
        NOT_IN_WRITE_OBJECT = new OutputStreamHook.DefaultState();
        IN_WRITE_OBJECT = new OutputStreamHook.InWriteObjectState();
        WROTE_DEFAULT_DATA = new OutputStreamHook.WroteDefaultDataState();
        WROTE_CUSTOM_DATA = new OutputStreamHook.WroteCustomDataState();
    }
    
    private class HookPutFields extends ObjectOutputStream.PutField
    {
        private Map<String, Object> fields;
        
        private HookPutFields() {
            this.fields = new HashMap();
        }
        
        @Override
        public void put(String s, boolean value) {
            this.fields.put(s, new Boolean(value));
        }
        
        @Override
        public void put(String s, char value) {
            this.fields.put(s, new Character(value));
        }
        
        @Override
        public void put(String s, byte value) {
            this.fields.put(s, new Byte(value));
        }
        
        @Override
        public void put(String s, short value) {
            this.fields.put(s, new Short(value));
        }
        
        @Override
        public void put(String s, int value) {
            this.fields.put(s, new Integer(value));
        }
        
        @Override
        public void put(String s, long value) {
            this.fields.put(s, new Long(value));
        }
        
        @Override
        public void put(String s, float value) {
            this.fields.put(s, new Float(value));
        }
        
        @Override
        public void put(String s, double value) {
            this.fields.put(s, new Double(value));
        }
        
        @Override
        public void put(String s, Object o) {
            this.fields.put(s, o);
        }
        
        @Override
        public void write(ObjectOutput objectOutput) throws IOException {
            OutputStreamHook outputStreamHook = (OutputStreamHook)objectOutput;
            ObjectStreamField[] fieldsNoCopy = outputStreamHook.getFieldsNoCopy();
            for (int i = 0; i < fieldsNoCopy.length; ++i) {
                outputStreamHook.writeField(fieldsNoCopy[i], this.fields.get(fieldsNoCopy[i].getName()));
            }
        }
    }
    
    protected static class WriteObjectState
    {
        public void enterWriteObject(OutputStreamHook outputStreamHook) throws IOException {
        }
        
        public void exitWriteObject(OutputStreamHook outputStreamHook) throws IOException {
        }
        
        public void defaultWriteObject(OutputStreamHook outputStreamHook) throws IOException {
        }
        
        public void writeData(OutputStreamHook outputStreamHook) throws IOException {
        }
    }
    
    protected static class DefaultState extends OutputStreamHook.WriteObjectState
    {
        @Override
        public void enterWriteObject(OutputStreamHook outputStreamHook) throws IOException {
            outputStreamHook.setState(OutputStreamHook.IN_WRITE_OBJECT);
        }
    }
    
    protected static class InWriteObjectState extends OutputStreamHook.WriteObjectState
    {
        @Override
        public void enterWriteObject(OutputStreamHook outputStreamHook) throws IOException {
            throw new IOException("Internal state failure: Entered writeObject twice");
        }
        
        @Override
        public void exitWriteObject(OutputStreamHook outputStreamHook) throws IOException {
            outputStreamHook.getOrbStream().write_boolean(false);
            if (outputStreamHook.getStreamFormatVersion() == 2) {
                outputStreamHook.getOrbStream().write_long(0);
            }
            outputStreamHook.setState(OutputStreamHook.NOT_IN_WRITE_OBJECT);
        }
        
        @Override
        public void defaultWriteObject(OutputStreamHook outputStreamHook) throws IOException {
            outputStreamHook.getOrbStream().write_boolean(true);
            outputStreamHook.setState(OutputStreamHook.WROTE_DEFAULT_DATA);
        }
        
        @Override
        public void writeData(OutputStreamHook outputStreamHook) throws IOException {
            outputStreamHook.getOrbStream().write_boolean(false);
            outputStreamHook.beginOptionalCustomData();
            outputStreamHook.setState(OutputStreamHook.WROTE_CUSTOM_DATA);
        }
    }
    
    protected static class WroteDefaultDataState extends OutputStreamHook.InWriteObjectState
    {
        @Override
        public void exitWriteObject(OutputStreamHook outputStreamHook) throws IOException {
            if (outputStreamHook.getStreamFormatVersion() == 2) {
                outputStreamHook.getOrbStream().write_long(0);
            }
            outputStreamHook.setState(OutputStreamHook.NOT_IN_WRITE_OBJECT);
        }
        
        @Override
        public void defaultWriteObject(OutputStreamHook outputStreamHook) throws IOException {
            throw new IOException("Called defaultWriteObject/writeFields twice");
        }
        
        @Override
        public void writeData(OutputStreamHook outputStreamHook) throws IOException {
            outputStreamHook.beginOptionalCustomData();
            outputStreamHook.setState(OutputStreamHook.WROTE_CUSTOM_DATA);
        }
    }
    
    protected static class WroteCustomDataState extends OutputStreamHook.InWriteObjectState
    {
        @Override
        public void exitWriteObject(OutputStreamHook outputStreamHook) throws IOException {
            if (outputStreamHook.getStreamFormatVersion() == 2) {
                ((ValueOutputStream)outputStreamHook.getOrbStream()).end_value();
            }
            outputStreamHook.setState(OutputStreamHook.NOT_IN_WRITE_OBJECT);
        }
        
        @Override
        public void defaultWriteObject(OutputStreamHook outputStreamHook) throws IOException {
            throw new IOException("Cannot call defaultWriteObject/writeFields after writing custom data in RMI-IIOP");
        }
        
        @Override
        public void writeData(OutputStreamHook outputStreamHook) throws IOException {
        }
    }
}
